
call sysproc.set_routine_opts('ISOLATION UR') @

CREATE OR REPLACE PROCEDURE RECLASSIFY_ALERT (
    IN I_TAB_ACRONYM CHARACTER(4),
    IN I_CMX_TRAN_ID BIGINT,
    IN I_ALERT_TYPE CHARACTER(1),
    IN I_PARENT_ALERT_TYPE CHARACTER(1),   --- If NO VALUE, there is no parent to set
    IN I_CONTACT_TYPE CHARACTER(1),        --- Will always have a value, but may not follow hierchary. The UI will validate the field is on the txn
    IN I_QUEUE_ID BIGINT,
    IN I_SRC_ALERT_ID BIGINT,
    IN I_SRC_ALERT_TYPE CHARACTER(1),
    IN I_SRC_ALERT_VERSION_NBR INTEGER,
    IN I_USER_NAME VARCHAR(60),
    IN I_NEED_TO_COMMIT	CHARACTER(1)
    )
SPECIFIC RECLASSIFY_ALERT
DYNAMIC RESULT SETS 1
LANGUAGE SQL
-----------------------------------------------------------------------------
-- file:  RECLASSIFY_ALERT
--
--
-- description:
--    Creates a new Alert based on an existing alert. The UI is validating the USER_NAME has access to the ALERT_TYPE
--    being created. The proc needs to validate access to the MO of the txn.
--1.	Look up the field name for the alert type.
--2.	Look up the field name for the parent alert type.
--3.	Look up the field name for the contact type
--6.	Look up  GMT Offset based on contact value. 
--7.	If alert does not exist, insert alert
--8.	If alert exists and not closed, do nothing
--9.	If alert exists and is closed, reinit
--10.	Insert alert action
--11.	Return alert ID on success
--12.   Update txn with marking
--13.   Insert fmx_fraud_updt
--14.	Throw exception on failure.
--
-- calls:	   
--     get_alert
--     set_xact_fraud_flag
--     save_alert_and_action
--
-- notes:
--
--
-- usage:
--		Called from the web UI
--
--
-- history:
--       2012/06/08    leushe      Created for Version 3.3
--     2014/03/17    leushe      Added reference for FRAUD_SCORE_2-4 and remove FRAUD_REASON_CODES
--     2015/01/27    LEUSHE     set FINAL_DISPOSITION_TIMESTAMP=null when performing the update on the alert
------------------------------------------------------------------------
P1: BEGIN
    DECLARE SQLCODE INTEGER DEFAULT 0;
    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';
	DECLARE L_SQLSTATE CHAR(5) DEFAULT '00000';
	DECLARE L_SQLCODE INTEGER;
    DECLARE L_TENANT_MULTI_ORG_ID INTEGER;
    DECLARE L_MO_NODE_KEY VARCHAR(20);
    DECLARE L_ROW_COUNT INTEGER DEFAULT -1;
    DECLARE L_CURRENT_UTC_TIMESTAMP TIMESTAMP;
    DECLARE L_ALERT_ACTION_STATUS VARCHAR(10) DEFAULT '';
    DECLARE L_ALERT_FIELD VARCHAR(30);
    DECLARE L_PARENT_ALERT_FIELD VARCHAR(30);
    DECLARE L_CONTACT_FIELD VARCHAR(30);
    DECLARE L_TABNAME VARCHAR(50);
    DECLARE L_ALERT_ID BIGINT DEFAULT -1;
    DECLARE L_CMX_TRAN_ID BIGINT;
    DECLARE L_ALERT_VERSION_NBR INTEGER DEFAULT 0;
    DECLARE L_ALERT_STATUS VARCHAR(10) DEFAULT 'ERROR' ;
    DECLARE L_BASESQL VARCHAR(2500);
    DECLARE L_INSERTSQL VARCHAR(4000);
    DECLARE L_UPDATESQL VARCHAR(4000);

    DECLARE INVALIDMOACCESS CONDITION FOR SQLSTATE 'SASFM';
    DECLARE ALERTCREATEFAILED CONDITION FOR SQLSTATE 'SASF2';
    DECLARE ALERTUPDATEFAILED CONDITION FOR SQLSTATE 'SASF3';
    
    DECLARE L_STMT STATEMENT;
    DECLARE L_INSERTSTMT STATEMENT;
    DECLARE L_UPDATESTMT STATEMENT;
    
    DECLARE c_stmt CURSOR for L_STMT;
    DECLARE c_insertStmt CURSOR for L_INSERTSTMT;
    DECLARE C_UPDATESTMT CURSOR for L_UPDATESTMT;
    
    
    DECLARE EXIT HANDLER FOR invalidMOAccess
       RESIGNAL SQLSTATE 'SASFM' SET MESSAGE_TEXT = 'User Does Not Have access to the Multi Org Id of the transaction';
    DECLARE EXIT HANDLER FOR alertCreateFailed
       RESIGNAL  SQLSTATE 'SASFM' SET MESSAGE_TEXT = 'Failed to Create new Alert';
    DECLARE EXIT HANDLER FOR alertUpdateFailed
       RESIGNAL  SQLSTATE 'SASFM' SET MESSAGE_TEXT = 'Failed to Re-Init Alert';
    --
    --  logic overview...
    --
	--1.	Look up the field name for the alert type.
	--2.	Look up the field name for the parent alert type.
	--3.	Look up the field name for the contact type
	--5.	Get cmx_client_amt and cmx_mod_amt from the transaction
	--6.	Look up  GMT Offset based on contact value. 
	--7.	If alert does not exist, insert alert
	--8.	If alert exists and not closed, return alert
	--9.	If alert exists and is closed, reinit
	--10.	Insert alert action
	--10b   Mark User Marking on the transaction for I_ALERT_TYPE for all cases (new, re-init'd,do nothing)
	--11.	Return alert ID on success or even if don't touch the alert
	--12.	Throw exception on failure.
	   
              
       --Set the current timestamp once
	   SET L_CURRENT_UTC_TIMESTAMP = CURRENT TIMESTAMP - CURRENT TIMEZONE;
       --Step 1
       select field_name into l_alert_field
       from fcm_field_definition, fcm_alert_type
       where key_field_id = field_id
       and alert_type = i_alert_type;
	   --Step 2
	   select field_name into l_parent_alert_field
       from fcm_field_definition, fcm_alert_type
       where key_field_id = field_id
       and alert_type = i_parent_alert_type;
	   --Step 3
	   select field_name into l_contact_field
       from fcm_field_definition, fcm_alert_type
       where key_field_id = field_id
       and alert_type = i_contact_type;
       
       --Get Txn data
       select sor_table_name into l_tabname 
	   from fcm_grid_trans_activity
	   where trans_activity_acronym = I_TAB_ACRONYM;
	  
    /*validate that the txn exists 
     * 				 the user has access to the MO 
     *              and the alertable field has a value in the TXN */
    set l_baseSql = 'select cmx_tran_id,smh_multi_org_node_key from '||l_tabname||' where cmx_tran_id = ? '
              ||' AND CHECK_MO_ACCESS((select user_id from fcm_user where user_name=? ),SMH_MULTI_ORG_ID) = 1 '
              ||' and length(trim('||l_alert_field||')) > 0 ';

    PREPARE l_stmt FROM l_baseSql;
		OPEN c_stmt using i_cmx_tran_id,i_user_name;
		fetch from c_stmt into l_cmx_tran_id,l_mo_node_key;
		SET l_sqlstate = SQLSTATE;
--		   if (l_sqlstate != '00000') then
--            	SIGNAL invalidMOAccess;
--   		   END IF;
 
    close c_stmt;
 
 
          
    /*if we didn't set l_cmx_tran_id in the validation query above, bail*/
    if (l_cmx_tran_id is NOT null) THEN
    set l_baseSql = 'select alert_id ,alert_version_nbr,alert_status from FCM_ALERT where alert_type = ? and alert_value = (select '||l_alert_field||' from '||l_tabname||' where cmx_tran_id = ? )';
   
      
    PREPARE l_stmt FROM l_baseSql;
		OPEN c_stmt using i_alert_type,i_cmx_tran_id;
		fetch from c_stmt into l_alert_id,l_alert_version_nbr,l_alert_status;
		SET l_sqlstate = SQLSTATE;
    close c_stmt;
    

	    select tenant_multi_org_id into l_TENANT_multi_org_id from fcm_multi_org where multi_org_node_key = l_mo_node_key;
		IF (l_TENANT_multi_org_id is null) then
	    	-- Set the min tenant multi_org_id into the local variable
       		select min(tenant_multi_org_id) into l_TENANT_multi_org_id from FCM_MULTI_ORG ;
		END IF;
	    
	    
   		if (l_sqlstate = '00000') then
            if (l_alert_status = 'CLOSED') then
                set l_alert_version_nbr=l_alert_version_nbr+1;
				        set l_alert_action_status = 'INITCLOSED';

  			    set l_updateSql= 'select alert_id from new table(UPDATE FCM_ALERT '
				     ||' SET LSTUPDT_NBR=LSTUPDT_NBR+1, '
				     ||'  (TENANT_MULTI_ORG_ID,SRC_ALERT_ID,SRC_ALERT_VERSION_NBR,SRC_ALERT_TYPE,INIT_CMX_TRAN_ID, INIT_SMH_ACCT_TYPE, INIT_SMH_ACTIVITY_TYPE,CONTACT_VALUE, PARENT_ALERT_VALUE,CONTACT_TYPE,PARENT_ALERT_TYPE,RESULT_ID,ALERT_STATUS,ALERT_ORIGIN,BASE_RULE_ID,RULE_VERSION_NBR,ALERT_SERVICE_CNT,MULTI_ORG_ID, '
				     ||'       FINAL_DISPOSITION_TIMESTAMP, LST_TRANSACTION_TIMESTAMP, INIT_TRANSACTION_TIMESTAMP,LST_TRANSACTION_AMT,INIT_TRANSACTION_AMT,LST_TRANSACTION_MOD_AMT,INIT_TRANSACTION_MOD_AMT,FRAUD_SCORE_1,HWM_FRAUD_SCORE_1, '
				     ||' 	   INIT_FRAUD_SCORE_1,FRAUD_SCORE_2,HWM_FRAUD_SCORE_2,INIT_FRAUD_SCORE_2,FRAUD_SCORE_3,HWM_FRAUD_SCORE_3,INIT_FRAUD_SCORE_3,'
				     ||' 	   FRAUD_SCORE_4,HWM_FRAUD_SCORE_4,INIT_FRAUD_SCORE_4,'
				     ||' 	   PAUSE_TIMESTAMP,PAUSE_ELAPSED_TIME,QUEUE_ID,ALERT_VERSION_NBR,CHECKOUT_TIMESTAMP,CREATE_USER,CREATE_TIMESTAMP,LSTUPDT_USER,LSTUPDT_TIMESTAMP ) '
				     ||' 	= (SELECT ?,?,?,?,cmx_tran_id,smh_acct_type,smh_activity_type,'
				     ||l_contact_field||', '
				     ||coalesce(l_parent_alert_field,'null')||', ' 
				     ||' 		?,?,NULL,''ACTIVE'',''INBOUND'',NULL,NULL,0,SMH_MULTI_ORG_ID,NULL,timestamp(coalesce(rqo_tran_date,''01-01-1960''),coalesce(rqo_tran_time,''00:00:00'')), '
				     ||' 		timestamp(coalesce(rqo_tran_date,''01-01-1960''),coalesce(rqo_tran_time,''00:00:00'')),CMX_CLIENT_AMT,CMX_CLIENT_AMT,CMX_MOD_AMT,CMX_MOD_AMT, '
				     ||' 		RRR_SCORE_1,RRR_SCORE_1,RRR_SCORE_1,RRR_SCORE_2,RRR_SCORE_2,RRR_SCORE_2,RRR_SCORE_3,RRR_SCORE_3,RRR_SCORE_3,RRR_SCORE_4,RRR_SCORE_4,RRR_SCORE_4,NULL,NULL,?,?,NULL,?,?, '
				     ||' 		?,? '
				     ||' FROM '||l_tabname
				     ||' 	WHERE CMX_TRAN_ID = ? ) '
				     ||' WHERE ALERT_ID= ?	)';							
	      		PREPARE l_updateStmt FROM l_updateSql;
			    OPEN c_updateStmt using L_TENANT_MULTI_ORG_ID, I_SRC_ALERT_ID,I_SRC_ALERT_VERSION_NBR,I_SRC_ALERT_TYPE,i_contact_type,I_parent_alert_type,I_QUEUE_ID,l_alert_version_nbr,I_USER_NAME,L_CURRENT_UTC_TIMESTAMP, 
				     	I_USER_NAME,L_CURRENT_UTC_TIMESTAMP,i_cmx_tran_id,l_ALERT_ID;
			      fetch from c_updateStmt into l_alert_id;
			        set l_sqlcode=SQLCODE;
           		SET l_sqlstate = SQLSTATE;
   		   		if (l_sqlstate != '00000') then
            		SIGNAL alertUpdateFailed;
   		   		END IF;

        		
     	   		close c_updateStmt;     					
 													
            end if;
   		
   		else
   		    set l_alert_version_nbr=0;
   		    set l_alert_action_status = 'ACTIVE';
   			set l_insertSql= 'select alert_id from new table(INSERT into fcm_alert (TENANT_MULTI_ORG_ID,ALERT_VERSION_NBR, ALERT_VALUE, ALERT_TYPE, MULTI_ORG_ID, CONTACT_VALUE, PARENT_ALERT_VALUE, QUEUE_ID, ALERT_STATUS, GMT_OFFSET, FRAUD_SCORE_1, FRAUD_SCORE_2, FRAUD_SCORE_3, '
            ||' FRAUD_SCORE_4, INIT_FRAUD_SCORE_1, INIT_FRAUD_SCORE_2, INIT_FRAUD_SCORE_3, INIT_FRAUD_SCORE_4, HWM_FRAUD_SCORE_1, HWM_FRAUD_SCORE_2, HWM_FRAUD_SCORE_3, '
            ||' HWM_FRAUD_SCORE_4, ALERT_SERVICE_CNT, ALERT_ORIGIN, INIT_TRANSACTION_TIMESTAMP, INIT_TRANSACTION_AMT, LST_TRANSACTION_AMT, LST_TRANSACTION_TIMESTAMP, INIT_TRANSACTION_MOD_AMT, LST_TRANSACTION_MOD_AMT, '
            ||' NEXT_SERVICE_TIMESTAMP, QUEUE_ENTRY_TIMESTAMP, CREATE_USER, CREATE_TIMESTAMP, LSTUPDT_USER, LSTUPDT_TIMESTAMP, LSTUPDT_NBR, LOGICAL_DEL_IND, INIT_CMX_TRAN_ID, INIT_SMH_ACCT_TYPE, INIT_SMH_ACTIVITY_TYPE, CONTACT_TYPE, '
            ||' PARENT_ALERT_TYPE,SRC_ALERT_ID,SRC_ALERT_VERSION_NBR,SRC_ALERT_TYPE) select '
					  ||' ?,?, '||l_alert_field||', '
					  ||' ?, '
			      ||' SMH_MULTI_ORG_ID, ' 
			      ||l_contact_field||', '
			      ||coalesce(l_parent_alert_field,'null')||', ' 
			      ||' ? , '
			      ||' ''ACTIVE'', '
			      ||'   coalesce( (select gmt_offset  '
			      ||'       from fcm_contact  '
			      ||'       where contact_type = ? and contact_value = '||l_contact_field||'),0),  '
			      ||'    rrr_score_1,  '
			      ||'    rrr_score_2,  '
			      ||'    rrr_score_3,  '
			      ||'    rrr_score_4,  '
			      ||'    rrr_score_1,  '
			      ||'    rrr_score_2,  '
			      ||'    rrr_score_3,  '
			      ||'    rrr_score_4,  '
			      ||'    rrr_score_1,  '
			      ||'    rrr_score_2,  '
			      ||'    rrr_score_3,  '
			      ||'    rrr_score_4,  '
			      ||'    0,  '
			      ||'    ''INBOUND'',       '     
			      ||'    timestamp(coalesce(rqo_tran_date,''01-01-1960''),coalesce(rqo_tran_time,''00:00:00'')), '
			      ||'    cmx_client_amt, '
			      ||'    cmx_client_amt,  '
			      ||'    timestamp(coalesce(rqo_tran_date,''01-01-1960''),coalesce(rqo_tran_time,''00:00:00'')),  '
			      ||'    cmx_mod_amt,  '
			      ||'    cmx_mod_amt,  '
			      ||'    ?,  '
			      ||'    ?,  '
			      ||'    ?,  '
			      ||'    ?,  '
			      ||'    ?,  '
			      ||'    ?,  '
			      ||'    0,  '
			      ||'   ''0'',  '
			      ||'    cmx_tran_id,  '
			      ||'    SMH_ACCT_TYPE, ' 
			      ||'    SMH_ACTIVITY_TYPE, '
			      ||'    ?,  '
			      ||'    ?,  '
			      ||'    ?,  '
			      ||'    ?,  '
			      ||'    ? '
			||' from '||l_tabname||' where smh_acct_type = substr(?,1,2)  '
			||' and smh_activity_type = substr(?,3,2)  '
			||' and cmx_tran_id = ? )';

      	PREPARE l_insertStmt FROM l_insertSql;
		    OPEN c_insertStmt using l_TENANT_MULTI_ORG_ID,l_alert_version_nbr,i_alert_type,i_QUEUE_ID,i_contact_type,
		    		L_CURRENT_UTC_TIMESTAMP,L_CURRENT_UTC_TIMESTAMP,I_USER_NAME,L_CURRENT_UTC_TIMESTAMP,
		    		I_USER_NAME,L_CURRENT_UTC_TIMESTAMP,i_contact_type,I_parent_alert_type,I_SRC_ALERT_ID,I_SRC_ALERT_VERSION_NBR,I_SRC_ALERT_TYPE,I_TAB_ACRONYM,
		    		I_TAB_ACRONYM,i_cmx_tran_id;
		    fetch from c_insertStmt into l_alert_id;
		    
		   		   
           set l_sqlcode=SQLCODE;
           SET l_sqlstate = SQLSTATE;
   		   if (l_sqlstate != '00000') then
            	SIGNAL alertCreateFailed;
   		   END IF;
 
        close c_insertStmt;
        
    end if;
	
    IF (length(L_ALERT_ACTION_STATUS) > 0) THEN
    		 call save_alert_and_action (l_alert_id,L_ALERT_ACTION_STATUS,'N');
    END IF;
	
	call set_xact_fraud_flag (l_alert_id,l_alert_version_nbr,0,I_TAB_ACRONYM,i_alert_Type,i_user_name,
		cast(i_cmx_tran_id as varchar(50)),'1','N');
	
     IF (I_NEED_TO_COMMIT = 'Y') THEN
     	COMMIT;
     END IF;
     
     CALL GET_ALERT(L_ALERT_ID, I_USER_NAME);
     -- eventually will need to signal an exception; now just return an empty set
  ELSE
     CALL GET_ALERT(-1, I_USER_NAME);
  END IF;
  
END P1 @

