CREATE OR REPLACE PROCEDURE RECLASSIFY_ALERT (
     I_TAB_ACRONYM IN VARCHAR2,
     I_CMX_TRAN_ID IN NUMBER,
     I_ALERT_TYPE IN VARCHAR2,
     I_PARENT_ALERT_TYPE IN VARCHAR2,   --- If NO VALUE, there is no parent to set
     I_CONTACT_TYPE IN VARCHAR2,        --- Will always have a value, but may not follow hierchary. The UI will validate the field is on the txn
     I_QUEUE_ID IN NUMBER,
     I_SRC_ALERT_ID IN NUMBER,
     I_SRC_ALERT_TYPE IN VARCHAR2,
     I_SRC_ALERT_VERSION_NBR IN INTEGER,
     I_USER_NAME IN VARCHAR2,
     I_NEED_TO_COMMIT    IN VARCHAR2,
     o_getalertcur OUT SYS_REFCURSOR
    )

-----------------------------------------------------------------------------
-- file:  RECLASSIFY_ALERT
--
--
-- description:
--    Creates a new Alert based on an existing alert. The UI is validating the USER_NAME has access to the ALERT_TYPE
--    being created. The proc needs to validate access to the MO of the txn.
--1.    Look up the field name for the alert type.
--2.    Look up the field name for the parent alert type.
--3.    Look up the field name for the contact type
--6.    Look up  GMT Offset based on contact value. 
--7.    If alert does not exist, insert alert
--8.    If alert exists and not closed, do nothing
--9.    If alert exists and is closed, reinit
--10.    Insert alert action
--11.    Return alert ID on success
--12.   Update txn with marking
--13.   Insert fmx_fraud_updt
--14.    Throw exception on failure.
--
-- calls:       
--     get_alert
--     set_xact_fraud_flag
--     save_alert_and_action
--
-- notes:
--
--
-- usage:
--        Called from the web UI
--
--
-- history:
--     2012/07/23    leushe                 Ported from DB2
--     2014/03/17    leushe      Added reference for FRAUD_SCORE_2-4 and remove FRAUD_REASON_CODES
--     2015/01/27    LEUSHE     set FINAL_DISPOSITION_TIMESTAMP=null when performing the update on the alert
--	   2016/02/03    leushe     Catch the exception when the Tenant_multi_org is not found. Defaults Tenant_multi_org_id = 1
------------------------------------------------------------------------
is

     SQLSTATE VARCHAR2(5) DEFAULT '00000';
     L_SQLSTATE VARCHAR2(5) DEFAULT '00000';
     L_MO_NODE_KEY VARCHAR2(20);
     L_ROW_COUNT INTEGER DEFAULT -1;
     L_CURRENT_UTC_TIMESTAMP TIMESTAMP;
     L_ALERT_ACTION_STATUS VARCHAR2(10) DEFAULT '';
     L_ALERT_FIELD VARCHAR2(30);
     L_PARENT_ALERT_FIELD VARCHAR2(30);
     L_CONTACT_FIELD VARCHAR2(30);
     L_TENANT_MULTI_ORG_ID INTEGER;
     L_TABNAME VARCHAR2(50);
     L_ALERT_ID NUMBER(19) DEFAULT -1;
     L_CMX_TRAN_ID NUMBER(19);
     L_ALERT_VERSION_NBR INTEGER DEFAULT 0;
     L_ALERT_STATUS VARCHAR2(10) DEFAULT 'ERROR' ;
     L_BASESQL VARCHAR2(2500);
     L_INSERTSQL VARCHAR2(4000);
     L_UPDATESQL VARCHAR2(4000);

 INVALIDMOACCESS EXCEPTION;
  ALERTUPDATEFAILED EXCEPTION;
   ALERTCREATEFAILED EXCEPTION;

    

     TYPE refcur IS REF CURSOR;
      c_stmt refcur;
     TYPE refcurinsert IS REF CURSOR;
      c_insertStmt refcurinsert;

       

    --
    --  logic overview...
    --
    --1.    Look up the field name for the alert type.
    --2.    Look up the field name for the parent alert type.
    --3.    Look up the field name for the contact type
    --5.    Get cmx_client_amt and cmx_mod_amt from the transaction
    --6.    Look up  GMT Offset based on contact value. 
    --7.    If alert does not exist, insert alert
    --8.    If alert exists and not closed, return alert
    --9.    If alert exists and is closed, reinit
    --10.    Insert alert action
    --10b   Mark User Marking on the transaction for I_ALERT_TYPE for all cases (new, re-init'd,do nothing)
    --11.    Return alert ID on success or even if don't touch the alert
    --12.    Throw exception on failure.
       
 BEGIN    
       
              
       --Set the current timestamp once
        L_CURRENT_UTC_TIMESTAMP := SYS_EXTRACT_UTC(CURRENT_TIMESTAMP);
       --Step 1
       select field_name into l_alert_field
       from fcm_field_definition, fcm_alert_type
       where key_field_id = field_id
       and alert_type = i_alert_type;
       --Step 2
       if (i_parent_alert_type is not null) Then
       select field_name into l_parent_alert_field
       from fcm_field_definition, fcm_alert_type
       where key_field_id = field_id
       and alert_type = i_parent_alert_type;
       else
           l_parent_alert_field := null;
           end if;
       --Step 3
       if (i_contact_type is not null) Then
           select field_name into l_contact_field
           from fcm_field_definition, fcm_alert_type
           where key_field_id = field_id
           and alert_type = i_contact_type;
       else
           l_contact_field := null;
       end if;
       if (length(trim(l_contact_field))=0) then
        l_contact_Field:=null;
        end if;
       
       --Get Txn data
       select sor_table_name into l_tabname 
       from fcm_grid_trans_activity
       where trans_activity_acronym = I_TAB_ACRONYM;
      
    /*validate that the txn exists and the user has access to the MO of the TXN */
     l_baseSql := 'select cmx_tran_id, smh_multi_org_node_key from '||l_tabname||' where cmx_tran_id = :1 '
              ||' AND CHECK_MO_ACCESS((select user_id from fcm_user where user_name=:2 ),SMH_MULTI_ORG_ID) = 1 '
              ||' and length(trim('||l_alert_field||')) > 0 ';

    OPEN c_stmt FOR l_baseSql USING i_cmx_tran_id,i_user_name;
        fetch  c_stmt into l_cmx_tran_id,l_mo_node_key;
        IF (c_stmt%FOUND) THEN
                  l_sqlstate   := '00000';
        ELSE
                  l_sqlstate   := '99999';
        END IF;
    close c_stmt;
 
 
          
    /*if we didn't set l_cmx_tran_id in the validation query above, bail*/
    if (l_cmx_tran_id is NOT null) THEN
     l_baseSql := 'select alert_id ,alert_version_nbr,alert_status from FCM_ALERT where alert_type = :1 and alert_value = (select '||l_alert_field||' from '||l_tabname||' where cmx_tran_id = :2 )';
   
      
 
        OPEN c_stmt FOR l_baseSql USING  i_alert_type,i_cmx_tran_id;
        fetch  c_stmt into l_alert_id,l_alert_version_nbr,l_alert_status;
        IF (c_stmt%FOUND) THEN
                  l_sqlstate   := '00000';
        ELSE
                  l_sqlstate   := '99999';
        END IF;
        close c_stmt;

		BEGIN
        select tenant_multi_org_id into l_TENANT_multi_org_id from fcm_multi_org where multi_org_node_key = l_mo_node_key;
            EXCEPTION
          WHEN NO_DATA_FOUND THEN
                l_TENANT_multi_org_id:=1;
        END;
      

           if (l_sqlstate = '00000') then
            if (l_alert_status = 'CLOSED') then
                 l_alert_version_nbr:=l_alert_version_nbr+1;
                 l_alert_action_status := 'INITCLOSED';

           
                   l_updateSql:= 'UPDATE FCM_ALERT '
                     ||' SET LSTUPDT_NBR=LSTUPDT_NBR+1, '
                     ||'  (TENANT_MULTI_ORG_ID,SRC_ALERT_ID,SRC_ALERT_VERSION_NBR,SRC_ALERT_TYPE,INIT_CMX_TRAN_ID, INIT_SMH_ACCT_TYPE, INIT_SMH_ACTIVITY_TYPE,CONTACT_VALUE, PARENT_ALERT_VALUE,CONTACT_TYPE,PARENT_ALERT_TYPE,RESULT_ID,ALERT_STATUS,ALERT_ORIGIN,BASE_RULE_ID,RULE_VERSION_NBR,ALERT_SERVICE_CNT,MULTI_ORG_ID, ' 
                     ||'       FINAL_DISPOSITION_TIMESTAMP,LST_TRANSACTION_TIMESTAMP, INIT_TRANSACTION_TIMESTAMP,LST_TRANSACTION_AMT,INIT_TRANSACTION_AMT,LST_TRANSACTION_MOD_AMT,INIT_TRANSACTION_MOD_AMT,FRAUD_SCORE_1,HWM_FRAUD_SCORE_1, '
				     ||' 	   INIT_FRAUD_SCORE_1,FRAUD_SCORE_2,HWM_FRAUD_SCORE_2,INIT_FRAUD_SCORE_2,FRAUD_SCORE_3,HWM_FRAUD_SCORE_3,INIT_FRAUD_SCORE_3,'
				     ||' 	   FRAUD_SCORE_4,HWM_FRAUD_SCORE_4,INIT_FRAUD_SCORE_4,'
                     ||'        PAUSE_TIMESTAMP,PAUSE_ELAPSED_TIME,QUEUE_ID,ALERT_VERSION_NBR,CHECKOUT_TIMESTAMP,CREATE_USER,CREATE_TIMESTAMP,LSTUPDT_USER,LSTUPDT_TIMESTAMP ) '
                     ||'     = (SELECT :1,:2,:3,:4,cmx_tran_id,smh_acct_type,smh_activity_type,'
                     ||coalesce(l_contact_field,'null')||', '
                     ||coalesce(l_parent_alert_field,'null')||', '  
                     ||'         :5,:6,NULL,''ACTIVE'',''INBOUND'',NULL,NULL,0,SMH_MULTI_ORG_ID, NULL '
                     ||'         ,(case when (rqo_tran_date is null or rqo_tran_Time is null) then rqo_proc_utc_datetime else to_timestamp(to_char(rqo_tran_date)||'' ''||(TO_CHAR(rqo_tran_time,''hh:mi:ss''))) end) '
                     ||'         ,(case when (rqo_tran_date is null or rqo_tran_Time is null) then rqo_proc_utc_datetime else to_timestamp(to_char(rqo_tran_date)||'' ''||(TO_CHAR(rqo_tran_time,''hh:mi:ss''))) end) '
                     ||'         ,CMX_CLIENT_AMT,CMX_CLIENT_AMT,CMX_MOD_AMT,CMX_MOD_AMT '
					 ||' 		 ,RRR_SCORE_1,RRR_SCORE_1,RRR_SCORE_1,RRR_SCORE_2,RRR_SCORE_2,RRR_SCORE_2,RRR_SCORE_3,RRR_SCORE_3,RRR_SCORE_3,RRR_SCORE_4,RRR_SCORE_4,RRR_SCORE_4,NULL,NULL,:6,:7,NULL,:8,:9, '
                     ||'         :10,:11 '
                     ||' FROM '||l_tabname
                     ||'     WHERE CMX_TRAN_ID = :12 ) '
                     ||' WHERE ALERT_ID= :13    ';                            

                  execute immediate l_updateSql using  l_TENANT_multi_org_id,I_SRC_ALERT_ID,I_SRC_ALERT_VERSION_NBR,I_SRC_ALERT_TYPE,i_contact_type,I_parent_alert_type,I_QUEUE_ID,l_alert_version_nbr,I_USER_NAME,L_CURRENT_UTC_TIMESTAMP, 
                         I_USER_NAME,L_CURRENT_UTC_TIMESTAMP,i_cmx_tran_id,l_ALERT_ID;
                  
                  if (SQL%ROWCOUNT=1) THEN
                    l_sqlstate   := '00000';
               ELSE
                  RAISE alertUpdateFailed;
               END IF;

                         
                                                     
            end if;
           
           else

                               
                l_alert_version_nbr:=0;
                select alert_seq.nextval into l_alert_id from dual;
                l_alert_action_status := 'ACTIVE';
                l_insertSql:= 'INSERT into fcm_alert (TENANT_MULTI_ORG_ID,ALERT_VERSION_NBR, ALERT_VALUE, ALERT_TYPE, MULTI_ORG_ID, CONTACT_VALUE, PARENT_ALERT_VALUE, QUEUE_ID, ALERT_STATUS, GMT_OFFSET, FRAUD_SCORE_1,FRAUD_SCORE_2,FRAUD_SCORE_3,FRAUD_SCORE_4, '
            ||' INIT_FRAUD_SCORE_1,INIT_FRAUD_SCORE_2,INIT_FRAUD_SCORE_3,INIT_FRAUD_SCORE_4, HWM_FRAUD_SCORE_1,HWM_FRAUD_SCORE_2,HWM_FRAUD_SCORE_3,HWM_FRAUD_SCORE_4,'
            ||' ALERT_SERVICE_CNT, ALERT_ORIGIN, INIT_TRANSACTION_TIMESTAMP, INIT_TRANSACTION_AMT, LST_TRANSACTION_AMT, LST_TRANSACTION_TIMESTAMP, INIT_TRANSACTION_MOD_AMT, LST_TRANSACTION_MOD_AMT, '
            ||' NEXT_SERVICE_TIMESTAMP, QUEUE_ENTRY_TIMESTAMP, CREATE_USER, CREATE_TIMESTAMP, LSTUPDT_USER, LSTUPDT_TIMESTAMP, LSTUPDT_NBR, LOGICAL_DEL_IND, INIT_CMX_TRAN_ID, INIT_SMH_ACCT_TYPE, INIT_SMH_ACTIVITY_TYPE, CONTACT_TYPE, '
            ||' PARENT_ALERT_TYPE,SRC_ALERT_ID,SRC_ALERT_VERSION_NBR,SRC_ALERT_TYPE,ALERT_ID) select '
                      ||' :1,:2, '||l_alert_field||', '
                      ||' :3, '
                  ||' SMH_MULTI_ORG_ID, ' 
                  ||coalesce(l_contact_field,'null')||', '
                  ||coalesce(l_parent_alert_field,'null')||', '  
                  ||' :4 , '
                  ||' ''ACTIVE'', '
                  ||'   0 ,'
			      ||'    rrr_score_1,  '
			      ||'    rrr_score_2,  '
			      ||'    rrr_score_3,  '
			      ||'    rrr_score_4,  '
			      ||'    rrr_score_1,  '
			      ||'    rrr_score_2,  '
			      ||'    rrr_score_3,  '
			      ||'    rrr_score_4,  '
			      ||'    rrr_score_1,  '
			      ||'    rrr_score_2,  '
			      ||'    rrr_score_3,  '
			      ||'    rrr_score_4,  '
			      ||'    0,  '
                  ||'    ''INBOUND'',       '     
                  ||'    (case when (rqo_tran_date is null or rqo_tran_Time is null) then rqo_proc_utc_datetime else to_timestamp(to_char(rqo_tran_date)||'' ''||(TO_CHAR(rqo_tran_time,''hh:mi:ss''))) end) ,'                  
                  ||'    cmx_client_amt, '
                  ||'    cmx_client_amt,  '
                  ||'    (case when (rqo_tran_date is null or rqo_tran_Time is null) then rqo_proc_utc_datetime else to_timestamp(to_char(rqo_tran_date)||'' ''||(TO_CHAR(rqo_tran_time,''hh:mi:ss''))) end) ,'
                  ||'    cmx_mod_amt,  '
                  ||'    cmx_mod_amt,  '
                  ||'    :5,  '
                  ||'    :6,  '
                  ||'    :7,  '
                  ||'    :8,  '
                  ||'    :9,  '
                  ||'    :10,  '
                  ||'    0,  '
                  ||'   ''0'',  '
                  ||'    cmx_tran_id,  '
                  ||'    SMH_ACCT_TYPE, ' 
                  ||'    SMH_ACTIVITY_TYPE, '
                  ||'    :11,  '
                  ||'    :12,  '
                  ||'    :13,  '
                  ||'    :14,  '
                  ||'    :15, '
                  ||'    :16 '
            ||' from '||l_tabname||' where '
            ||'  cmx_tran_id = :17 ';

 
            execute immediate l_insertSql using l_TENANT_multi_org_id,l_alert_version_nbr,i_alert_type,i_QUEUE_ID,
                    L_CURRENT_UTC_TIMESTAMP,L_CURRENT_UTC_TIMESTAMP,I_USER_NAME,L_CURRENT_UTC_TIMESTAMP,
                    I_USER_NAME,L_CURRENT_UTC_TIMESTAMP,i_contact_type,I_parent_alert_type,I_SRC_ALERT_ID,I_SRC_ALERT_VERSION_NBR,I_SRC_ALERT_TYPE,l_alert_id,i_cmx_tran_id;
             if (SQL%ROWCOUNT=1) THEN
                    l_sqlstate   := '00000';
             ELSE
                  RAISE alertCreateFailed;
             END IF;
  

        
    end if;
    

   
    
   
       
    IF (length(L_ALERT_ACTION_STATUS) > 0) THEN
              save_alert_and_action (l_alert_id,L_ALERT_ACTION_STATUS,'N');
    END IF;
    
     set_xact_fraud_flag (l_alert_id,l_alert_version_nbr,0,I_TAB_ACRONYM,i_alert_Type,i_user_name,
        i_cmx_tran_id ,'1','N');
    
     IF (I_NEED_TO_COMMIT = 'Y') THEN
         COMMIT;
     END IF;
     
     GET_ALERT(l_alert_id, i_user_name,o_getalertcur);
     -- eventually will need to signal an exception; now just return an empty set
  ELSE
     GET_ALERT(-1, i_user_name,o_getalertcur);
  END IF;
    EXCEPTION
          WHEN invalidMOAccess THEN
                RAISE_APPLICATION_ERROR(-1, 'User Does Not Have access to the Multi Org Id of the transaction');
          WHEN alertCreateFailed THEN
                RAISE_APPLICATION_ERROR(-1, 'Failed to Create new Alert');
         WHEN alertUpdateFailed THEN
                RAISE_APPLICATION_ERROR(-1, 'Failed to Re-Init Alert');
    
END RECLASSIFY_ALERT ;
/
